/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../state/reducers";
import Spinner from "../../helpers/spinner";
import {
  loadTutorViewSite,
  refreshUserAssessmentData,
  setBotLeftPanelViewSize,
  setBotRightPanelViewSize,
  setBottomLeftSelectedTabIndex,
  setBottomRightSelectedTabIndex,
  setDoExerciseHDividerPosOffset,
  setDoExerciseVDividerPosOffset,
  setSelectedConsoleViewTest,
  setTopLeftPanelViewSize,
  setTopLeftSelectedTabIndex,
  setTopRightPanelViewSize,
  setTopRightSelectedTabIndex,
  setTutorViewDummy,
} from "../../../state/actions/tutorViewSite/tutorViewSiteActions";
import PanelGrid from "../../panelSystem/panelGrid/panelGrid";
import TabContentHost from "../../panelSystem/panel/tabContentHost";
import Panel from '../../panelSystem/panel/panel'
import {RouteComponentProps, withRouter} from "react-router";
import {parseBoolean, parseId} from "../../../helpers/convertersAndTransformers";

import PreviewPanelView from '../exerciseEditorSite/previewPanel/previewPanelView'

import TestsPanelView from '../doExerciseSite/testsPanel/testsPanelView'

import ConsolePanelView from '../doExerciseSite/consolePanel/consolePanelView'
import ConsolePanelHeaderBar from '../doExerciseSite/consolePanel/consolePanelHeaderBar'
import {AssessedSubmitTestFrontendExclusive} from "../../../types/testResults";
import {RunnableCustomTestFrontendExclusive} from "../../../types/customTests";
import {RunnableExerciseTestFrontendExclusive} from "../../../types/exerciseEditor";
import {
  cancelViewTestDialog,
  openViewTestDialog
} from "../../../state/actions/tutorViewSite/dialogs/viewTestDialog/viewTestDialogActions";
import ViewTestDialog from '../doExerciseSite/testsPanel/viewTestDialog/viewTestDialog'
import {debugOriginUrl} from "../../../../debug";
import SubmitTestsPanelHeaderBar from '../doExerciseSite/submitTestsPanel/submitTestsPanelHeaderBar'
import EditorSettingsDialog from '../../editorSettingsDialog/editorSettingsDialog'

import UserSolutionEditorPanelHeaderBar from './userSolutionEditorPanel/editorPanelHeaderBar'
import UserSolutionEditorPanel from './userSolutionEditorPanel/editorPanelView'

import AfterEditorPanelView from './userAfterSolutionEditorPanel/afterEditorPanelView'
import AfterEditorPanelHeaderBar from './userAfterSolutionEditorPanel/afterEditorPanelHeaderBar'

import TutorEditorPanelHeaderBar from './tutorEditorPanel/editorPanelHeaderBar'
import TutorEditorPanel from './tutorEditorPanel/editorPanelView'
import {
  runNormalTestAsTutor,
  runSubmitTestAsTutor
} from "../../../state/actions/tutorViewSite/tutorViewSiteCrudActions";
import {RunTutorTestCommandForBackend} from "../../../types/testsAndTestProtocol";

import NormalTestsForTutorPanelHeaderBar from './normalTestsPanel/normalTestsPanelHeaderBar'
import SubmitTestsForTutorPanelHeaderBar from './submitTestsPanel/submitTestsPanelHeaderBar'

import ManualAssessmentPanelView from './manualAssessmentPanel/manualAssessmentPanelView'


import {errorDialog} from "../../../helpers/dialogHelper";
import {LoginTimeoutHelper} from "../../../helpers/loginTimeoutHelper";
import {createAndRefreshAssetUrlsAndObjects, freeAssetUrlsAndObjects} from "../../../helpers/assetHelper";
import {getI18n} from "../../../../i18n/i18nRoot";
import {ErrorHelper} from '../../../helpers/errorHelper'
import ZoomPreviewPanelActions from '../exerciseEditorSite/previewPanel/zoomPreviewPanelActions'
import {AceEditorsStorageHelper} from '../../codeEditors/aceEditors/aceEditorsStorageHelper'
import {tutorViewUserSolutionEditorPanelViewEditorId} from '../../../constants'
import CodeTemplateEditorPanelHeaderBar from './codeTemplateEditorPanel/codeTemplateEditorPanelHeaderBar'
import CodeTemplateEditorPanel from './codeTemplateEditorPanel/codeTemplateEditorPanelView'
import MarkdownRenderer from '../../renderContentComponents/markdownRenderer'
import {QuestionType} from '../../../types/questions'
import FeedbackPanelHeaderBar from './feedbackPanel/feedbackPanelHeaderBar'
import FeedbackEditorView from './feedbackPanel/feedbackEditorView'

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
}

const mapStateToProps = (rootState: RootState /*, props: MyProps*/) => {
  return {
    //test0: rootState...
    //test: props.test

    exercise: rootState.tutorViewSiteState.exercise,

    feedbackForStudentPanelSelectedSubTabIndex: rootState.tutorViewSiteState.feedbackForStudentPanelSelectedSubTabIndex,

    taskDescriptionFontSizeInPx: rootState.globalUiState.taskDescriptionFontSizeInPx,

    normalTestsForTutor: rootState.tutorViewSiteState.normalTestsForTutor,
    submitTestsForTutor: rootState.tutorViewSiteState.submitTestsForTutor,

    afterNormalTests: rootState.tutorViewSiteState.afterNormalTests,
    afterSubmitTests: rootState.tutorViewSiteState.afterSubmitTests,

    submitTestResults: rootState.tutorViewSiteState.submitTestResults,
    userSolution: rootState.tutorViewSiteState.userSolution,
    userSolutionPLangId: rootState.tutorViewSiteState.userSolution.pLangId,

    assessment: rootState.tutorViewSiteState.assessment,

    userAfterSolution: rootState.tutorViewSiteState.userAfterSolution,

    tutorEditorSolution: rootState.tutorViewSiteState.tutorEditorSolution,

    testTypes: rootState.testTypesState.testTypes,

    selectedConsoleViewTest: rootState.tutorViewSiteState.selectedConsoleViewTest,


    isNoDialogDisplayed:
      !rootState.tutorViewSiteState.isViewTestDialogDisplayed
      && !rootState.tutorViewSiteState.isCodeEditorSettingsDialogDisplayed,

    topRightSelectedTabIndex: rootState.tutorViewSiteState.topRightSelectedTabIndex,
    topLeftSelectedTabIndex: rootState.tutorViewSiteState.topLeftSelectedTabIndex,
    bottomRightSelectedTabIndex: rootState.tutorViewSiteState.bottomRightSelectedTabIndex,
    bottomLeftSelectedTabIndex: rootState.tutorViewSiteState.bottomLeftSelectedTabIndex,

    topRightPanelViewSize: rootState.tutorViewSiteState.topRightPanelViewSize,
    topLeftPanelViewSize: rootState.tutorViewSiteState.topLeftPanelViewSize,
    botRightPanelViewSize: rootState.tutorViewSiteState.botRightPanelViewSize,
    botLeftPanelViewSize: rootState.tutorViewSiteState.botLeftPanelViewSize,

    hDividerPosOffset: rootState.tutorViewSiteState.hDividerPosOffset,
    vDividerPosOffset: rootState.tutorViewSiteState.vDividerPosOffset,

    isSomeTestRunning: rootState.tutorViewSiteState.exercise.tests.some(p => p.uiIsRunning)
      || rootState.tutorViewSiteState.exercise.customTests.some(p => p.uiIsRunning)
      || rootState.tutorViewSiteState.normalTestsForTutor.some(p => p.uiIsRunning)
      || rootState.tutorViewSiteState.submitTestsForTutor.some(p => p.uiIsRunning)
    ,

    comesFromOwnExercisesSite: rootState.tutorViewSiteState.comesFromOwnExercisesSite,

    isViewTestDialogDisplayed: rootState.tutorViewSiteState.isViewTestDialogDisplayed,
    isCodeEditorSettingsDialogDisplayed: rootState.tutorViewSiteState.isCodeEditorSettingsDialogDisplayed,

    isLoading: rootState.tutorViewSiteState.isLoading
      || rootState.testTypesState.isLoading
      || rootState.pLangsState.isLoading
      || rootState.globalSystemSettingsState.isLoading,

    releaseFinishedAndAutomaticAssessmentHasFinished: !rootState.tutorViewSiteState.exercise.exerciseRelease.isReleased &&
      rootState.tutorViewSiteState.exercise.exerciseRelease.hasAutomaticAssessmentFinished,

    dummy: rootState.tutorViewSiteState.dummy,
    langId: rootState.i18nState.langId
  }
}

const mapDispatchToProps = (dispatch: Dispatch) => bindActionCreators({
                                                                        //imported reducer funcs here
                                                                        setDoExerciseHDividerPosOffset,
                                                                        setDoExerciseVDividerPosOffset,
                                                                        setTopLeftSelectedTabIndex,
                                                                        setTopLeftPanelViewSize,
                                                                        setTopRightSelectedTabIndex,
                                                                        setTopRightPanelViewSize,
                                                                        setBottomLeftSelectedTabIndex,
                                                                        setBotLeftPanelViewSize,
                                                                        setBottomRightSelectedTabIndex,
                                                                        setBotRightPanelViewSize,

                                                                        loadTutorViewSite,
                                                                        refreshUserAssessmentData,
                                                                        setTutorViewDummy,
                                                                        setSelectedConsoleViewTest,
                                                                        openViewTestDialog,
                                                                        cancelViewTestDialog,

                                                                        runNormalTestAsTutor,
                                                                        runSubmitTestAsTutor,
                                                                      }, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

interface MatchedProps {
  readonly comesFromOwnExercisesSite: string | 'null' //null = assessment statistic
  readonly releaseId: string
  readonly userId: string
  readonly pLangId: string
}

const tutorExerciseTaskPrintId = 'tutor-exercise-task-print-id'
const tutorFeedbackForStudentPintId = 'tutor-feedback-for-student-pint-id'

class tutorViewSite extends React.Component<Props & RouteComponentProps<MatchedProps>, any> {


  async componentWillMount(): Promise<void> {
    await this.refreshData(this.props, true)
  }

  async refreshData(props: Props & RouteComponentProps<MatchedProps>, initialRender: boolean): Promise<void> {

    const releaseId = parseId(props.match.params.releaseId)
    const userId = parseId(props.match.params.userId)
    const pLangId = parseId(props.match.params.pLangId)
    const comesFromOwnExercisesSite = parseBoolean(props.match.params.comesFromOwnExercisesSite)

    if (releaseId === null || userId === null || pLangId === null || (comesFromOwnExercisesSite === null && props.match.params.comesFromOwnExercisesSite !== 'null')
    ) {
      errorDialog(
        getI18n(props.langId, 'Error'),
        getI18n(props.langId, 'Some url parameters were not found/set'), props.langId
      )
      return
    }

    const viewId = `${releaseId}-${userId}-${pLangId}`

    if (LoginTimeoutHelper.isReLogin && viewId === LoginTimeoutHelper.lastKnownTutorAssessmentId) {
      //do not reload or reset site state...
      //user re logged in he/she want's to continue
      //if we would reload then the code would be gone (even though the frontend settings would store a copy... but that's bad style)

      //unmount cleared everything...
      createAndRefreshAssetUrlsAndObjects(props.exercise.exerciseDescription.assets)
      setTutorViewDummy(false)

      return

    } else {
      LoginTimeoutHelper.lastKnownTutorAssessmentId = viewId
    }


    //reset ace storage else when we open the next submission (via button) the editor is not changed
    AceEditorsStorageHelper.removeAll()
    try {

      if (initialRender) {
        await props.loadTutorViewSite(releaseId, userId, pLangId, comesFromOwnExercisesSite)
      } else {
        //we used next/previous buttons we only need to get some new data (not all)
        await props.refreshUserAssessmentData(releaseId, userId, pLangId, comesFromOwnExercisesSite)
      }

    } catch (err) {
      ErrorHelper.makeServerErrorDialog(err, this.props.langId, 'Error', 'Could not load site data')
    }

  }


  componentWillReceiveProps(nextProps: Props & RouteComponentProps<MatchedProps>, nextContext: any): void {


    if (this.props.location.pathname === nextProps.location.pathname) return


    const releaseId = parseId(nextProps.match.params.releaseId)
    const userId = parseId(nextProps.match.params.userId)
    const pLangId = parseId(nextProps.match.params.pLangId)
    const comesFromOwnExercisesSite = parseBoolean(nextProps.match.params.comesFromOwnExercisesSite)

    if (releaseId === null || userId === null || pLangId === null || (comesFromOwnExercisesSite === null && nextProps.match.params.comesFromOwnExercisesSite !== 'null')
    ) {
      errorDialog(
        getI18n(nextProps.langId, 'Error'),
        getI18n(nextProps.langId, 'Some url parameters were not found/set'), nextProps.langId
      )
      return
    }


    //we only need a full refresh if the exercise release changed
    //for a different user we only need to load some new data and can keep the e.g. exercise
    if (this.props.exercise.exerciseRelease.id !== releaseId
    // || this.props.userSolution.userId !== userId
    // || this.props.userSolution.pLangId !== pLangId
    // || this.props.comesFromOwnExercisesSite !== comesFromOwnExercisesSite
    ) {
      //full reload
      this.refreshData(nextProps, true)
      return
    }

    //only load new user solution & assessment
    this.refreshData(nextProps, false)

  }

  componentWillUnmount(): void {
    //if we not freeing we would leak memory... if we just exit the site
    freeAssetUrlsAndObjects()
    setTutorViewDummy(true) //enable re rendering
  }

  render(): JSX.Element {
    return (
      <div className="fh">

        {
          this.props.isLoading &&
          <Spinner/>
        }

        {
          this.props.isViewTestDialogDisplayed &&
          <ViewTestDialog
            cancelViewTestDialog={() => {
              this.props.cancelViewTestDialog()
            }}
            getDownloadTestAssetLink={(testId: number, fileId: number, isSubmitTest: boolean) => {
              return `${debugOriginUrl}/api/assets/${this.props.exercise.exerciseRelease.releaseCode}/${testId}/${fileId}/${isSubmitTest}/true`
            }}
          />
        }


        {
          this.props.isCodeEditorSettingsDialogDisplayed &&
          <EditorSettingsDialog/>
        }

        {
          this.props.isNoDialogDisplayed &&
          <PanelGrid
            width="100%"
            height="100%"

            topRightPanelViewSize={this.props.topRightPanelViewSize}
            topLeftPanelViewSize={this.props.topLeftPanelViewSize}
            botRightPanelViewSize={this.props.botRightPanelViewSize}
            botLeftPanelViewSize={this.props.botLeftPanelViewSize}

            hDividerPosOffset={this.props.hDividerPosOffset}
            vDividerPosOffset={this.props.vDividerPosOffset}
            onHDividerPosOffsetChanged={(Offset) => {
              this.props.setDoExerciseHDividerPosOffset(Offset)
            }}
            onVDividerPosOffsetChanged={(Offset) => {
              this.props.setDoExerciseVDividerPosOffset(Offset)
            }}

            topLeftPanel={
              <Panel
                applyDefaultPadding={false}
                selectedTabIndex={this.props.topLeftSelectedTabIndex}
                onSelectedTabChanged={(index) => {
                  this.props.setTopLeftSelectedTabIndex(index)
                }}

                onSizeChangedClicked={this.props.setTopLeftPanelViewSize}
                panelViewSize={this.props.topLeftPanelViewSize}
              >
                <TabContentHost
                  title={getI18n(this.props.langId, 'Tutor-Editor')}
                  icon={'font'}
                  applyDefaultPadding={false}
                  defaultHelpText={getI18n(this.props.langId,
                                           "This editor is used to check and assess the submission manually. It's also used to run the tests. Any modification is NOT saved to the server."
                  )}
                  tabHeaderBar={
                    <TutorEditorPanelHeaderBar/>
                  }
                >
                  <TutorEditorPanel/>
                </TabContentHost>

                <TabContentHost
                  title={getI18n(this.props.langId, 'Submission code')}
                  icon={'send outline'}
                  applyDefaultPadding={false}
                  defaultHelpText={getI18n(this.props.langId,
                                           'This editor displays the original submission in a readonly editor.'
                  )}
                  tabHeaderBar={
                    <UserSolutionEditorPanelHeaderBar/>
                  }
                >
                  <UserSolutionEditorPanel/>
                </TabContentHost>

                {
                  this.props.userAfterSolution !== null &&
                  <TabContentHost
                    title={getI18n(this.props.langId, 'Subsequent editor code')}
                    icon={'life ring'}
                    applyDefaultPadding={false}
                    defaultHelpText={getI18n(this.props.langId,
                                             'This editor displays the subsequent editor code in a readonly editor.'
                    )}
                    tabHeaderBar={
                      <AfterEditorPanelHeaderBar/>
                    }
                  >
                    <AfterEditorPanelView/>
                  </TabContentHost>
                }


              </Panel>
            }

            topRightPanel={
              <Panel
                selectedTabIndex={this.props.topRightSelectedTabIndex}
                onSelectedTabChanged={(index) => {
                  this.props.setTopRightSelectedTabIndex(index)
                }}
                onSizeChangedClicked={this.props.setTopRightPanelViewSize}
                panelViewSize={this.props.topRightPanelViewSize}
                additionalActionsRightIcons={
                  //only allow zoom for exercise task description and feedback for student (but only the preview!)
                  (this.props.topRightSelectedTabIndex === 0 || (this.props.topRightSelectedTabIndex === 2 && this.props.feedbackForStudentPanelSelectedSubTabIndex === 1))
                  ? <ZoomPreviewPanelActions printId={this.props.topRightSelectedTabIndex === 0
                                                      ? tutorExerciseTaskPrintId
                                                      : tutorFeedbackForStudentPintId} canPrint={true}/>
                  : undefined
                }
              >
                <TabContentHost
                  title={getI18n(this.props.langId, 'Task description')}
                  icon={'align left'}
                >

                  <PreviewPanelView printId={tutorExerciseTaskPrintId}
                                    descriptionContent={this.props.exercise.exerciseDescription.content}
                                    assets={this.props.exercise.exerciseDescription.assets}
                                    dummy={this.props.dummy}
                                    fontSizeInPx={this.props.taskDescriptionFontSizeInPx}
                  />

                </TabContentHost>

                <TabContentHost
                  title={getI18n(this.props.langId, 'Manual assessment')}
                  icon={'flag'}
                  iconClass={this.props.topRightSelectedTabIndex !== 1 &&
                             ((this.props.assessment.noteForOtherTutors !== null && this.props.assessment.noteForOtherTutors.length > 0) ||
                               this.props.assessment.manualPoints !== null)
                             ? 'warning-colored'
                             : undefined}
                  defaultHelpText={getI18n(this.props.langId, "If you assign manual points or feedback for the student then the manual points-and-feedback-panel is displayed to the student.")}
                  tabHeaderBar={null}
                >
                  <ManualAssessmentPanelView/>
                </TabContentHost>

                <TabContentHost
                  title={getI18n(this.props.langId, 'Feedback for student')}
                  icon={'talk'}
                  iconClass={this.props.topRightSelectedTabIndex !== 2 && this.props.assessment.feedbackForStudent !== null && this.props.assessment.feedbackForStudent.length > 0
                             ? 'warning-colored'
                             : undefined}
                  defaultHelpText={getI18n(this.props.langId, "If you assign manual points or feedback for the student then the manual points-and-feedback-panel is displayed to the student.")}
                  tabHeaderBar={
                    <FeedbackPanelHeaderBar/>
                  }
                >
                  {
                    this.props.feedbackForStudentPanelSelectedSubTabIndex === 0 && this.props.releaseFinishedAndAutomaticAssessmentHasFinished &&
                    <FeedbackEditorView/>
                  }
                  {
                    (this.props.feedbackForStudentPanelSelectedSubTabIndex === 1 || this.props.releaseFinishedAndAutomaticAssessmentHasFinished === false) &&
                    <div>
                      <MarkdownRenderer
                        printId={tutorFeedbackForStudentPintId}
                        content={this.props.assessment.feedbackForStudent !== null
                                 ? this.props.assessment.feedbackForStudent
                                 : ''}
                        assets={[]}
                        contentType={QuestionType.markdown}
                        dummy={this.props.dummy}
                        fontSizeInPx={this.props.taskDescriptionFontSizeInPx}
                      />
                    </div>
                  }

                </TabContentHost>


                <TabContentHost title={getI18n(this.props.langId, "Code template")}
                                icon={'file code outline'}
                                tabHeaderBar={<CodeTemplateEditorPanelHeaderBar/>}
                >

                  <CodeTemplateEditorPanel/>
                </TabContentHost>

              </Panel>
            }

            bottomLeftPanel={
              <Panel
                selectedTabIndex={this.props.bottomLeftSelectedTabIndex}
                onSelectedTabChanged={(index) => {
                  this.props.setBottomLeftSelectedTabIndex(index)
                }}
                onSizeChangedClicked={this.props.setBotLeftPanelViewSize}
                panelViewSize={this.props.botLeftPanelViewSize}
              >
                {
                  //readonly test results
                }
                <TabContentHost
                  title={getI18n(this.props.langId, 'Normal tests')}
                  icon={'bug'}
                  tabHeaderBar={null}
                >
                  <TestsPanelView
                    areAfterTests={false}
                    displayWeightIndicatorsForResults={false}
                    tests={this.props.exercise.tests}
                    selectedConsoleViewTest={this.props.selectedConsoleViewTest}
                    isUserAllowedToRunTests={false}
                    editorHasUnsavedChanges={false}
                    isSomeTestRunning={false}
                    onSaveSolution={null}
                    setSelectedConsoleViewTest={(test: RunnableExerciseTestFrontendExclusive | RunnableCustomTestFrontendExclusive | AssessedSubmitTestFrontendExclusive | null) => {
                      this.props.setSelectedConsoleViewTest(test)
                    }}

                    onRunTest={null}
                    openViewTestDialog={(runnableTest: RunnableExerciseTestFrontendExclusive | AssessedSubmitTestFrontendExclusive | null) => {
                      this.props.openViewTestDialog(runnableTest)
                    }}
                  />

                </TabContentHost>

                <TabContentHost
                  title={getI18n(this.props.langId, 'Submit tests')}
                  icon={'flag checkered'}
                  tabHeaderBar={

                    this.props.releaseFinishedAndAutomaticAssessmentHasFinished &&
                    <SubmitTestsPanelHeaderBar submitTestResults={this.props.submitTestResults}/>

                  }
                  defaultHelpText={getI18n(this.props.langId,
                                           'The submit tests results are only displayed after the automatic assessment has finished. Note that these submit tests are executed on the submit test server (not the normal test server), so there might be different settings e.g. timeouts'
                  )}
                >

                  {
                    !this.props.releaseFinishedAndAutomaticAssessmentHasFinished &&
                    <div>
                      {
                        getI18n(this.props.langId,
                                'The release is still open or the automatic assessment has not finished yet'
                        )
                      }
                    </div>
                  }
                  {
                    this.props.releaseFinishedAndAutomaticAssessmentHasFinished &&
                    <TestsPanelView
                      areAfterTests={false}
                      displayWeightIndicatorsForResults={true}
                      tests={this.props.submitTestResults}
                      selectedConsoleViewTest={this.props.selectedConsoleViewTest}
                      isUserAllowedToRunTests={false}
                      editorHasUnsavedChanges={false}
                      isSomeTestRunning={false}
                      onSaveSolution={null}
                      setSelectedConsoleViewTest={(test: RunnableExerciseTestFrontendExclusive | RunnableCustomTestFrontendExclusive | AssessedSubmitTestFrontendExclusive | null) => {
                        this.props.setSelectedConsoleViewTest(test)
                      }}

                      onRunTest={null}
                      openViewTestDialog={(runnableTest: RunnableExerciseTestFrontendExclusive | AssessedSubmitTestFrontendExclusive | null) => {
                        this.props.openViewTestDialog(runnableTest)
                      }}
                    />
                  }

                </TabContentHost>

                {
                  // not readonly tests (so that the tutor don't change the result...
                }

                <TabContentHost
                  title={getI18n(this.props.langId, 'Tutor-Tests')}
                  icon={'bug'}
                  defaultHelpText={getI18n(this.props.langId,
                                           "These tests are run with the Tutor-Editor content. Results are only temporarily and not saved on the backend."
                  )}
                  tabHeaderBar={
                    <NormalTestsForTutorPanelHeaderBar/>
                  }
                >
                  <TestsPanelView
                    areAfterTests={false}
                    displayWeightIndicatorsForResults={false}
                    tests={this.props.normalTestsForTutor}
                    selectedConsoleViewTest={this.props.selectedConsoleViewTest}
                    isUserAllowedToRunTests={true}
                    editorHasUnsavedChanges={false}
                    isSomeTestRunning={this.props.isSomeTestRunning}
                    onSaveSolution={null}
                    setSelectedConsoleViewTest={(test: RunnableExerciseTestFrontendExclusive | RunnableCustomTestFrontendExclusive | AssessedSubmitTestFrontendExclusive | null) => {
                      this.props.setSelectedConsoleViewTest(test)
                    }}

                    onRunTest={(test: RunnableExerciseTestFrontendExclusive | RunnableCustomTestFrontendExclusive | AssessedSubmitTestFrontendExclusive) => {

                      const runTestAsTutorCommand: RunTutorTestCommandForBackend = {
                        releaseCode: this.props.exercise.exerciseRelease.releaseCode,
                        isSubmitTest: false,
                        testIds: [test.id],
                        isNormalTest: true,
                        plangId: this.props.tutorEditorSolution.pLangId,
                        mainFileId: this.props.tutorEditorSolution.mainFileId,
                        solutionFiles: this.props.tutorEditorSolution.solutionFiles
                      }
                      this.props.runNormalTestAsTutor(runTestAsTutorCommand)
                    }}
                    openViewTestDialog={(runnableTest: RunnableExerciseTestFrontendExclusive | AssessedSubmitTestFrontendExclusive | null) => {
                      this.props.openViewTestDialog(runnableTest)
                    }}
                  />

                </TabContentHost>

                <TabContentHost
                  title={getI18n(this.props.langId, 'Tutor-Submit-Tests')}
                  icon={'flag checkered'}
                  defaultHelpText={getI18n(this.props.langId,
                                           "These tests are run with the Tutor-Editor content. Results are only temporarily and not saved on the backend."
                  )}
                  tabHeaderBar={
                    this.props.releaseFinishedAndAutomaticAssessmentHasFinished &&
                    <SubmitTestsForTutorPanelHeaderBar/>
                  }
                >
                  {
                    !this.props.releaseFinishedAndAutomaticAssessmentHasFinished &&
                    <div>
                      {
                        getI18n(this.props.langId,
                                'The release is still open or the automatic assessment has not finished yet'
                        )
                      }
                    </div>
                  }
                  {
                    this.props.releaseFinishedAndAutomaticAssessmentHasFinished &&
                    <TestsPanelView
                      areAfterTests={false}
                      displayWeightIndicatorsForResults={true}
                      tests={this.props.submitTestsForTutor}
                      selectedConsoleViewTest={this.props.selectedConsoleViewTest}
                      isUserAllowedToRunTests={true}
                      editorHasUnsavedChanges={false}
                      isSomeTestRunning={this.props.isSomeTestRunning}
                      onSaveSolution={null}
                      setSelectedConsoleViewTest={(test: RunnableExerciseTestFrontendExclusive | RunnableCustomTestFrontendExclusive | AssessedSubmitTestFrontendExclusive | null) => {
                        this.props.setSelectedConsoleViewTest(test)
                      }}

                      onRunTest={(test: RunnableExerciseTestFrontendExclusive | RunnableCustomTestFrontendExclusive | AssessedSubmitTestFrontendExclusive) => {

                        const runTestAsTutorCommand: RunTutorTestCommandForBackend = {
                          releaseCode: this.props.exercise.exerciseRelease.releaseCode,
                          isSubmitTest: true,
                          testIds: [test.id],
                          isNormalTest: false,
                          plangId: this.props.tutorEditorSolution.pLangId,
                          mainFileId: this.props.tutorEditorSolution.mainFileId,
                          solutionFiles: this.props.tutorEditorSolution.solutionFiles
                        }
                        this.props.runSubmitTestAsTutor(runTestAsTutorCommand)
                      }}
                      openViewTestDialog={(runnableTest: RunnableExerciseTestFrontendExclusive | AssessedSubmitTestFrontendExclusive | null) => {
                        this.props.openViewTestDialog(runnableTest)
                      }}
                    />
                  }
                </TabContentHost>

                {
                  // after normal, submit tests from the user
                }

                {
                  this.props.userAfterSolution !== null &&
                  <TabContentHost
                    title={getI18n(this.props.langId, 'Normal tests')}
                    defaultHelpText={getI18n(this.props.langId,
                                             "The results from the subsequent solution normal tests"
                    )}
                    icon={'life ring'}
                    tabHeaderBar={null}
                  >
                    <TestsPanelView
                      areAfterTests={true}
                      displayWeightIndicatorsForResults={false}
                      tests={this.props.afterNormalTests}
                      selectedConsoleViewTest={this.props.selectedConsoleViewTest}
                      isUserAllowedToRunTests={false}
                      editorHasUnsavedChanges={false}
                      isSomeTestRunning={false}
                      onSaveSolution={null}
                      setSelectedConsoleViewTest={(test: RunnableExerciseTestFrontendExclusive | RunnableCustomTestFrontendExclusive | AssessedSubmitTestFrontendExclusive | null) => {
                        this.props.setSelectedConsoleViewTest(test)
                      }}

                      onRunTest={null}
                      openViewTestDialog={(runnableTest: RunnableExerciseTestFrontendExclusive | AssessedSubmitTestFrontendExclusive | null) => {
                        this.props.openViewTestDialog(runnableTest)
                      }}
                    />

                  </TabContentHost>
                }
                {
                  this.props.userAfterSolution !== null &&
                  <TabContentHost
                    title={getI18n(this.props.langId, 'Submit tests')}
                    icon={'life ring'}
                    tabHeaderBar={

                      this.props.releaseFinishedAndAutomaticAssessmentHasFinished &&
                      <SubmitTestsPanelHeaderBar submitTestResults={this.props.submitTestResults}/>

                    }
                    defaultHelpText={`${getI18n(this.props.langId,
                                                "These are the test results from the user with the subsequent editor content."
                    )}${getI18n(this.props.langId,
                                'The submit tests results are only displayed after the automatic assessment has finished. Note that these submit tests are executed on the submit test server (not the normal test server), so there might be different settings e.g. timeouts'
                    )}`}
                  >

                    {
                      !this.props.releaseFinishedAndAutomaticAssessmentHasFinished &&
                      <div>
                        {
                          getI18n(this.props.langId,
                                  'The release is still open or the automatic assessment has not finished yet'
                          )
                        }
                      </div>
                    }
                    {
                      this.props.releaseFinishedAndAutomaticAssessmentHasFinished &&
                      <TestsPanelView
                        areAfterTests={true}
                        displayWeightIndicatorsForResults={true}
                        tests={this.props.afterSubmitTests}
                        selectedConsoleViewTest={this.props.selectedConsoleViewTest}
                        isUserAllowedToRunTests={false}
                        editorHasUnsavedChanges={false}
                        isSomeTestRunning={false}
                        onSaveSolution={null}
                        setSelectedConsoleViewTest={(test: RunnableExerciseTestFrontendExclusive | RunnableCustomTestFrontendExclusive | AssessedSubmitTestFrontendExclusive | null) => {
                          this.props.setSelectedConsoleViewTest(test)
                        }}

                        onRunTest={null}
                        openViewTestDialog={(runnableTest: RunnableExerciseTestFrontendExclusive | AssessedSubmitTestFrontendExclusive | null) => {
                          this.props.openViewTestDialog(runnableTest)
                        }}
                      />
                    }

                  </TabContentHost>
                }
              </Panel>
            }

            bottomRightPanel={
              <Panel
                selectedTabIndex={this.props.bottomRightSelectedTabIndex}
                onSelectedTabChanged={(index) => {
                  this.props.setBottomRightSelectedTabIndex(index)
                }}
                onSizeChangedClicked={this.props.setBotRightPanelViewSize}
                panelViewSize={this.props.botRightPanelViewSize}
              >
                <TabContentHost
                  title={getI18n(this.props.langId, 'Console output')}
                  icon={'terminal'}
                  tabHeaderBar={
                    <ConsolePanelHeaderBar test={this.props.selectedConsoleViewTest} testTypes={this.props.testTypes}
                    />
                  }
                >
                  <ConsolePanelView test={this.props.selectedConsoleViewTest} protocolItems={null}
                                    testTypes={this.props.testTypes}
                                    displayFilterConsoleItemsOptions={true}
                  />
                </TabContentHost>
              </Panel>
            }
          />
        }

      </div>
    )
  }
}

export default withRouter(connect(mapStateToProps, mapDispatchToProps)(tutorViewSite))
